@@ -4,7 +4,7 @@ test_that("summarise_draws works correctly", {
   expect_true(all(default_convergence_measures() %in% names(sum_x)))
   expect_true(all(c("q5", "q95") %in% names(sum_x)))
   expect_equal(sum_x$variable, variables(x))
-  expect_equal(mean(x$mu), sum_x$mean[sum_x$variable == "mu"])
+  expect_equal(mean(x$mu), as.numeric(sum_x$mean[sum_x$variable == "mu"]))
 
   sum_x <- summarise_draws(x, mean, median)
   expect_true(all(c("mean", "median") %in% names(sum_x)))
@@ -120,6 +120,13 @@ test_that(paste(
   expect_identical(sum_x, parsum_x)
 })
 
+test_that("summarise_draws supports tibble::num correctly", {
+  x <- example_draws()
+  expect_output(
+    print(summarise_draws(x, .num_args = list(sigfig = 2, notation="dec"))),
+    "<dec:2>"
+  )
+})
 
 test_that("summarise_draws errors for invalid cores specification", {
   x <- example_draws()
@@ -132,3 +139,32 @@ test_that("summarise_draws errors for invalid cores specification", {
     "Cannot coerce '.cores' to a single integer value"
   )
 })
+
+
+test_that("summarise_draws works with variance()", {
+  draws_array <- as_draws_array(example_draws())
+  draws_matrix <- as_draws_matrix(draws_array)
+  draws_df <- as_draws_df(draws_array)
+  draws_list <- as_draws_list(draws_array)
+  draws_rvars <- as_draws_rvars(draws_array)
+
+  ref <- data.frame(
+    variable = variables(draws_array),
+    variance = tibble::num(as.vector(
+      apply(draws_array, 3, function(x) var(as.vector(x)))
+    )),
+    stringsAsFactors = FALSE
+  )
+  class(ref) <- posterior:::class_draws_summary()
+
+  expect_equal(summarise_draws(draws_array, variance), ref)
+  expect_equal(summarise_draws(draws_matrix, variance), ref)
+  expect_equal(summarise_draws(draws_df, variance), ref)
+  expect_equal(summarise_draws(draws_list, variance), ref)
+  expect_equal(summarise_draws(draws_rvars, variance), ref)
+
+  # for consistency, draws_matrix and draws_array
+  # have the same implementation of variance()
+  expect_equal(variance(draws_array), var(as.vector(draws_array)))
+  expect_equal(variance(draws_matrix), var(as.vector(draws_matrix)))
+})
